/* MODULE $clitab; */
#define block_c_vector	1
#define block_c_command	2
#define block_c_type	3
#define block_c_entity	4
#define block_c_expression	5
#define block_c_cdu_visited	6

/*
 * Vector at front of tables
 */
struct vector_block {
  unsigned short vec_w_size;
  unsigned char vec_b_type;
  unsigned char vec_b_subtype;
#define vec_c_dcl	1
#define vec_c_mcr	2
#define vec_c_verb	3
#define vec_c_command	4
  struct {
    unsigned char vec_b_strlvl;
#define vec_c_strlvl	6
    unsigned : 8;
  } vec_r_flags;
  unsigned short vec_w_tro_count;
#define vec_c_header_length	8
  long vec_l_verbtbl;
  long vec_l_comdptr;
  unsigned long vec_l_table_size;
#define vec_c_length	20
};

/*
 * Command descriptor block.
 */
struct command_block {
  unsigned short cmd_w_size;
  unsigned char cmd_b_type;
  unsigned char cmd_b_subtype;
#define cmd_c_verb	1
#define cmd_c_syntax	2
  struct {
    int cmd_v_abbrev : 1;
#define cmd_m_abbrev	1
    int cmd_v_nostat : 1;
#define cmd_m_nostat	2
    int cmd_v_foreign : 1;
#define cmd_m_foreign	4
    int cmd_v_immed : 1;
#define cmd_m_immed	8
    int cmd_v_mcrparse : 1;
#define cmd_m_mcrparse	16
    int cmd_v_parms : 1;
#define cmd_m_parms	32
    int cmd_v_quals : 1;
#define cmd_m_quals	64
    int cmd_v_disallows : 1;
#define cmd_m_disallows	128
    unsigned : 8;
  } cmd_r_flags;
  unsigned short cmd_w_tro_count;
  long cmd_l_parms;
#define cmd_c_max_parms	8
  long cmd_l_quals;
  long cmd_l_disallow;
  unsigned char cmd_b_handler;
#define cmd_c_none	0
#define cmd_c_cli	1
#define cmd_c_user	2
#define cmd_c_image	3
#define cmd_c_same	4
  struct {
    int cmd_v_minparm : 4;
    int cmd_v_maxparm : 4;
  } cmd_r_parmcnt;
  unsigned char cmd_b_verbtyp;
  char cmd_b_pad1;
  unsigned short cmd_w_name;
  unsigned short cmd_w_image;
  unsigned short cmd_w_outputs;
  unsigned short cmd_w_prefix;
#define cmd_c_length	32
  char cmd_z_variable[];
#define cmd_c_max_name	32
#define cmd_c_max_image	64
#define cmd_c_max_outputs	8
#define cmd_c_max_prefix	32
};

/*
 * Type descriptor block.
 */
struct type_block {
  unsigned short type_w_size;
  unsigned char type_b_type;
  unsigned char type_b_subtype;
#define type_c_type	1
  unsigned short type_w_flags;
  unsigned short type_w_tro_count;
  long type_l_keywords;
  unsigned short type_w_name;
  unsigned short type_w_prefix;
#define type_c_length	16
  char type_z_variable[];
#define type_c_max_name	32
#define type_c_max_prefix	32
};

/*
 * Entity descriptor block
 */
struct entity_block {
  unsigned short ent_w_size;
  unsigned char ent_b_type;
  unsigned char ent_b_subtype;
#define ent_c_parameter	1
#define ent_c_qualifier	2
#define ent_c_keyword	3
  struct {
    int ent_v_val : 1;
#define ent_m_val	1
    int ent_v_neg : 1;
#define ent_m_neg	2
    int ent_v_deftrue : 1;
#define ent_m_deftrue	4
    int ent_v_batdef : 1;
#define ent_m_batdef	8
    int ent_v_valreq : 1;
#define ent_m_valreq	16
    int ent_v_list : 1;
#define ent_m_list	32
    int ent_v_concat : 1;
#define ent_m_concat	64
    int ent_v_impcat : 1;
#define ent_m_impcat	128
    int ent_v_verb : 1;
#define ent_m_verb	256
    int ent_v_parm : 1;
#define ent_m_parm	512
    int ent_v_mcroptdelim : 1;
#define ent_m_mcroptdelim	1024
    int ent_v_mcrignore : 1;
#define ent_m_mcrignore	2048
    unsigned : 4;
  } ent_r_flags;
  unsigned short ent_w_tro_count;
  long ent_l_next;
  long ent_l_syntax;
  long ent_l_user_type;
  unsigned char ent_b_number;
  unsigned char ent_b_valtype;
#define ent_c_user_defined	0
#define ent_c_infile	1
#define ent_c_outfile	2
#define ent_c_number	3
#define ent_c_privilege	4
#define ent_c_datetime	5
#define ent_c_protection	6
#define ent_c_process	7
#define ent_c_inlog	8
#define ent_c_outlog	9
#define ent_c_insym	10
#define ent_c_outsym	11
#define ent_c_node	12
#define ent_c_device	13
#define ent_c_dir	14
#define ent_c_uic	15
#define ent_c_restofline	16
#define ent_c_parenvalue	17
#define ent_c_deltatime	18
#define ent_c_quotedstring	19
#define ent_c_file	20
#define ent_c_expression	21
#define ent_c_test1	22
#define ent_c_test2	23
#define ent_c_test3	24
#define ent_c_acl	25
#define ent_c_max_valtype	26
  unsigned short ent_w_name;
  unsigned short ent_w_label;
  unsigned short ent_w_prompt;
  unsigned short ent_w_defval;
#define ent_c_length	30
  char ent_z_variable[];
#define ent_c_max_name	32
#define ent_c_max_label	32
#define ent_c_max_prompt	32
#define ent_c_max_defval	96
};

/*
 * Expression block
 */
struct expression_block {
  unsigned short exp_w_size;
  unsigned char exp_b_type;
  unsigned char exp_b_subtype;
#define exp_c_path	1
#define exp_c_not	2
#define exp_c_any2	3
#define exp_c_and	4
#define exp_c_or	5
#define exp_c_xor	6
#define exp_c_neg	7
  unsigned short exp_w_flags;
  unsigned short exp_w_tro_count;
#define exp_c_length	8
  char exp_l_operand_list[];
#define exp_c_max_path_entities	8
};
/* END_MODULE; */
